'''
Created on 2011-01-17

@author: Chris
'''

import pygame
from pygame.locals import *
import os, sys
from player import *
from levelhandling import *
from dialoguemanager import *
from cutscenes import *
from loadtools import *

#set to 1 to turn on debug data while game runs
DEBUG_MODE = 0

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

def main():

    #define constants
    FPS_LIMIT = 25

    # buffer of space between the player and the colliding object
    EXPAND_VALUE = 10
    
    #amount that player bets on every machine
    BET_AMOUNT = 50
    
    #phrases that come up in dialogue box
    BET_PHRASE_START = 'Press <Enter> to bet $' + str(BET_AMOUNT) + ' normally or ... ' +\
        'press <c> to cheat ('
    BET_PHRASE_END = '% higher chance of winning)'
    FUNKY_PHRASE_START = 'Funky Gambling Behavior: '
    
    SCORE_FONT_SIZE = 36
    DIALOGUE_FONT_SIZE = 18
    
    #amount of money player needs to get to win the game
    PLAYER_MONEY_WIN = 30000 
    
    #number out of 150 that represents how funky your betting practices are
    funky_num = 0
    FUNKY_NUM_LIMIT = 150
    
    FUNKY_CHEAT_WIN = 2
    FUNKY_CHEAT_LOSE = 4
    FUNKY_REGULAR_WIN = 1
    FUNKY_REGULAR_LOSS = -1
    
    # contains recently collided object
    recent_collided_object_rect = None
    recent_collided_object = None
    
    # Initialise screen and clock
    pygame.init()
    screen = pygame.display.set_mode((800, 600))
    frame_clock = pygame.time.Clock()
    
    #Initialize player sprite
    player = Player(pygame.Color(25,0,0), (650,300), 'playerhorizwalk.png', \
                    'playervertwalkup.png','playervertwalkdown.png')
    
    #group relevant sprites
    sprite_group = pygame.sprite.Group(player)
    
    #load level area
    terrain_asset_index = AssetIndex('levelAssetIndex.txt')
    level_area = LevelArea('a0.txt', terrain_asset_index, screen)
    
    # store level area terrain background
    terrain_bg = level_area.get_terrain_bg()
    # add dialogue frame to terrain background
    dialogue_frame_image = pygame.image.load(os.path.join('data', \
            'dialogueframe.png')).convert_alpha()
    terrain_bg.blit(dialogue_frame_image,(0,450))
    
    #create text sprites
    score_text = HUDInfoSprite('$'+str(player.money),(700,550),SCORE_FONT_SIZE)
    dialogue_text = HUDInfoSprite("",(15,550),DIALOGUE_FONT_SIZE)
    funky_num_text = HUDInfoSprite(FUNKY_PHRASE_START + str(funky_num) + '/' +\
                                   str(FUNKY_NUM_LIMIT),(15,480),DIALOGUE_FONT_SIZE)
    
    #create cutscenes
    cutscene_intro = Cutscene('intro.png')
    cutscene_win = Cutscene('win.png')
    cutscene_fail = Cutscene('fail.png')
    
    # efficiently create a sprite group containing the player sprite
    player_sprite_group = pygame.sprite.RenderUpdates()
    player_sprite_group.add(player)
    
    
    # efficiently create a sprite group containing all object sprites
    object_sprite_group = pygame.sprite.RenderUpdates()
    for sprite in level_area.get_object_sprite_list():
        object_sprite_group.add(sprite)
        
        
    #load dialogue box
    #hud = HUD('dialogueframe.png')
    
    # efficiently create a sprite group containing the dialogue manager sprite
    #hud_sprite_group = pygame.sprite.RenderUpdates()
    #hud_sprite_group.add(hud)
    
    
    #efficiently create a sprite group containing the text sprites
    text_sprite_group = pygame.sprite.RenderUpdates()
    text_sprite_group.add(score_text)
    text_sprite_group.add(dialogue_text)
    text_sprite_group.add(funky_num_text)

    
    #show intro cutscene
    cutscene_intro.play(screen)

    # Blit initial frame to screen
    screen.blit(terrain_bg,(0,0))
    pygame.display.flip()
    
    level_music = load_sound('levelmusic.WAV')
    level_music.play()
    
    #### Enter the game loop ####
    program_is_running = True
    while program_is_running:
        gameplay_mode = 1
        while gameplay_mode:
            #run the frame_clock and get the miliseconds since the last frame
            frame_duration = frame_clock.tick(FPS_LIMIT)
        
            player_movement = (frame_duration/1000.0) * player.movement_speed
             
            #process game controller inputs
            for event in pygame.event.get():
                if event.type == QUIT: quit()
            
                elif event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
                        quit()
                
                    elif event.key == K_LEFT:
                        player.horiz_velocity(-round(player_movement))
                        player.vert_velocity(0)
                
                    elif event.key == K_RIGHT:
                        player.horiz_velocity(round(player_movement))
                        player.vert_velocity(0)
                    
                    elif event.key == K_UP:
                        player.horiz_velocity(0)
                        player.vert_velocity(-round(player_movement))
                    
                    elif event.key == K_DOWN:
                        player.horiz_velocity(0)
                        player.vert_velocity(round(player_movement))
                    
                    elif event.key == K_RETURN or event.key == K_KP_ENTER:
                        if recent_collided_object != None:
                            # player bets normally
                            amount_won = recent_collided_object.bet(BET_AMOUNT)
                            player.money += amount_won
                            score_text.set_text('$'+str(player.money), SCORE_FONT_SIZE)
                            if amount_won > 0:
                                funky_num += FUNKY_REGULAR_WIN
                            else:
                                funky_num += FUNKY_REGULAR_LOSS
                            if funky_num < 0:
                                funky_num = 0
                            
                            funky_num_text.set_text(FUNKY_PHRASE_START + str(funky_num) + '/' +\
                                   str(FUNKY_NUM_LIMIT),DIALOGUE_FONT_SIZE)
                                
                    elif event.key == K_c:
                        if recent_collided_object != None:
                            # player bets with cheating
                            amount_won = recent_collided_object.bet(BET_AMOUNT,True)
                            player.money += amount_won
                            score_text.set_text('$'+str(player.money), SCORE_FONT_SIZE)
                            if amount_won > 0:
                                funky_num += FUNKY_CHEAT_WIN
                            else:
                                funky_num += FUNKY_CHEAT_WIN
                            
                            funky_num_text.set_text(FUNKY_PHRASE_START + str(funky_num) + '/' +\
                                   str(FUNKY_NUM_LIMIT),DIALOGUE_FONT_SIZE)
                                
                            
                    
                else:
                    player.horiz_velocity(0)
                    player.vert_velocity(0)              
                 
            #Update all sprite groups
            for sprite in object_sprite_group:
                sprite.update()
            for sprite in player_sprite_group:
                sprite.update()
            #for sprite in hud_sprite_group:
            #   sprite.update()
        
            #check for collisions with player and any of the game objects
            collided_objects = pygame.sprite.spritecollide(player, object_sprite_group, 0)
            if collided_objects != []:
                #print 'player colliding with object'
                # handle player movement with colliding object
                if player.x_movement != 0:
                    player.rect.left -= player.x_movement
                elif player.y_movement != 0:
                    player.rect.top -= player.y_movement
            
                recent_collided_object = collided_objects[0]
                recent_collided_object_rect = recent_collided_object.rect.copy()
                recent_collided_object_rect.inflate_ip(EXPAND_VALUE,EXPAND_VALUE)
                #print 'directly within gambling distance'
                dialogue_text.set_text(BET_PHRASE_START + \
                    recent_collided_object.get_cheat_increase_string() +\
                    BET_PHRASE_END,DIALOGUE_FONT_SIZE)

            if collided_objects == [] and recent_collided_object != None:
                if player.rect.colliderect(recent_collided_object_rect):
                    dialogue_text.set_text(BET_PHRASE_START + \
                        recent_collided_object.get_cheat_increase_string() +\
                        BET_PHRASE_END,DIALOGUE_FONT_SIZE)
                    #print 'within gambling distance'
                
                else:
                    dialogue_text.set_text('',DIALOGUE_FONT_SIZE)
                    recent_collided_object = None
                    recent_collided_object_rect = None
        
            # clear all sprites from the screen
            object_sprite_group.clear(screen, terrain_bg)
            player_sprite_group.clear(screen, terrain_bg)
            text_sprite_group.clear(screen, terrain_bg)
            #hud_sprite_group.clear(screen, terrain_bg)
        
            #draw all sprite groups to the screen
            dirty = object_sprite_group.draw(screen)
            dirty += player_sprite_group.draw(screen)
            dirty += text_sprite_group.draw(screen)
            #dirty += hud_sprite_group.draw(screen)
        
            # flip only the dirty areas of the screen, which avoids flipping the 
            # whole screen
            pygame.display.update(dirty)
            
            if player.money < 0 or funky_num > FUNKY_NUM_LIMIT:
                #Can play bad cutscene here
                gameplay_mode = 0
                #game over
                cutscene_fail.play(screen)
                
            elif player.money >= PLAYER_MONEY_WIN:
                #Can play good coutscene here
                gameplay_mode = 0
                #game over
                cutscene_win.play(screen)
                               
            
        #end of gameplay mode = reset values
        player.rect.topleft = (650,300)
        player.money = 25000
        funky_num = 0
        funky_num_text.set_text(FUNKY_PHRASE_START + str(funky_num) + '/' +\
                                   str(FUNKY_NUM_LIMIT),DIALOGUE_FONT_SIZE)
        score_text.set_text('$'+str(player.money), SCORE_FONT_SIZE)
        
        
        screen.blit(terrain_bg, (0,0))
        pygame.display.flip()
        
               
        
if __name__ == '__main__': main()