'''
Created on 2011-01-29

@author: Chris
'''

import sys, os
import io
import re
import pygame
from objects import *

class Level():
    """ a class that keeps track of our current map section in the world """
    
    def __init__(self, level_filename, terrain_asset_index, screen):
        file = open(os.path.join('data', 'maps', 'level', level_filename))
        self.level_area_layout = []
        self.level_area_dict = {}
        # the level file will have the ordering and names of the the map section 
        # files -- currently this class in NOT used in the game -- perhaps it will
        # be used in a more sophisticated version of the game
        entry_format = re.compile('([^\\s]+)*[^\\s]+')
        
        for line in file:
            if entry_format.match(line):
                self.level_area_layout.append([])
                split_line = line.split(' ')
                for area in split_line:
                    self.level_area_layout[len(self.level_area_layout)-1].append( \
                        area.strip())
                    try:
                        level_area = LevelArea(area.strip()+'.txt', \
                                               terrain_asset_index, screen)
                    except:
                        print 'problem loading level area ' + area.strip()
                        sys.exit()
                        
                    self.level_area_dict[area.strip()] = level_area
                    
            else:
                print 'there was a problem reading the level area file'
                sys.exit()
                
        #now we should have level_area_layout filled properly
        
        
        
        



# this is a class that manages information as it pertains to a section of the 
# game's overall map.  Note that only one section of the entire map is playable
# at a time

class LevelArea():
    """ a class that represents a single section of the game's map """
    
    def __init__(self, level_area_filename, asset_index, screen):
        """ each map section is tile-based and represented by a specially
        formatted text file """
        
        TERRAIN_TILE_SIZE = 50
        # each terrain tile is a square
        
        # the surface object that is the terrain background
        self.terrain_bg = pygame.Surface(screen.get_size())
        self.terrain_bg = self.terrain_bg.convert()
        
        # a list that will contain all of the sprite objects in this level
        self.object_sprites = []
        
        # check if map_sect_file is valid as we generate map area for this object
        map_section_file = open(os.path.join('data', 'maps', 'level', level_area_filename))
        max_horiz_terr_tiles = screen.get_size()[0]/TERRAIN_TILE_SIZE
        max_vert_terr_tiles = (screen.get_size()[1]-150)/TERRAIN_TILE_SIZE
        
        valid_object_line = re.compile('[^\\s\(\),]+\(\\d\\d*,\\d\\d*\)')
        
        #create terrain bg first
        mode = 'terrain'
        line_num = 0
        for line in map_section_file:
            
            if line == '\n':
                #after first blank line encountered, we go into object mode
                mode = 'object'
                continue
                
            
            if mode == 'terrain':
                if len(line) == (max_horiz_terr_tiles+1) and \
                    line_num < max_vert_terr_tiles:
                    
                    for i in range(0,len(line)):
                        tile = asset_index.get_tile(line[i])                  
                        self.terrain_bg.blit(tile, (i*TERRAIN_TILE_SIZE, \
                                                        line_num*TERRAIN_TILE_SIZE))
                    line_num += 1
                
                else:
                    print 'An error occured while trying to read the map section file \
                        in terrain mode'
                    sys.exit(1)

                    
            elif mode == 'object':
                if valid_object_line.match(line):
                    object_name = line[0:line.find('(')]
                    object_x_coord = int(line[(line.find('(')+1):line.find(',')])
                    object_y_coord = int(line[(line.find(',')+1):line.find(')')])
                    object_image = asset_index.get_object(object_name)
                    
                    #place object in object sprites map
                    # following code is not necessary right now -- may be used 
                    # or deleted later
                    """num = 1
                    if object_name in self.object_sprites:
                        object_name = object_name + '-' + str(num)
                        num += 1"""
                    #we now have a unique object name
                    
                    # right now we will use generic object sprites - but later on 
                    # we may use a switch statemet to determine which child of 
                    # sprite to store
                    if object_name == 'slotmachine':
                        object_sprite = CasinoGame((object_x_coord, object_y_coord), \
                                           object_image,0.20)
                    self.object_sprites.append(object_sprite)      
                                     
         
         # the terrain background has now been created
        
        
        
        
    
    def get_terrain_bg(self):
        """ a method that returns the terrain_bg surface """
        return self.terrain_bg
            
                    
                    
                    
    def get_object_sprite_list(self):
        """ returns the list containing all of the object sprites found in this
        level area """
        return self.object_sprites
        



        
        
class AssetIndex():
    """ a class that serves as a database to index characters in map section 
    files with their corresponding PNG images """
    
    
    def __init__(self, asset_filename):
        asset_file = open(os.path.join('data', 'maps', asset_filename))
        """ 
        terrain asset images should be placed in '<game root folder>/data/maps'
        """
        
        self.DEFAULT_TILE = pygame.Surface((50,50))
        self.DEFAULT_TILE.fill((255,20,147))
        
        self.DEFAULT_OBJECT = pygame.Surface((25,25))
        self.DEFAULT_OBJECT.fill((0,0,255))
        
        self.terrain_assets = {}
        self.object_assets = {}
        
        terrain_entry_format = re.compile('[^\\s]\\s=\\s[^\\s]+')
        object_entry_format = re.compile('[^\\s]+\\s:\\s[^\\s]+')
        
        for line in asset_file:
            """ enter all valid (character,tile image) pairs found in the terrain 
            asset index file into the in-game terrain_assets dictionary - any 
            invalid lines in the file are ignored
            """
            if terrain_entry_format.match(line):
                image_filename = line[4:-1]
                image = pygame.image.load(os.path.join('data', 'maps', 'terrain', \
                                                       image_filename)).convert_alpha()
                self.terrain_assets[line[0]] = image
                
            elif object_entry_format.match(line):
                separator_index = line.find(':')
                
                if separator_index != -1:
                    object_name = line[0:(separator_index - 1)]
                    image_filename = line[(separator_index + 2):-1]
                    image = pygame.image.load(os.path.join('data', 'maps', 'objects', \
                                                           image_filename)).convert_alpha()
                    self.object_assets[object_name] = image
                else:
                    print 'there was an error loading an object from the assets list'
                    sys.exit()

                
                
    
    def get_tile(self, char):
        """ Gets the tile image associated with the given character.
        If no tile image is found for the specified character, return a 
        default red tile. 
        
        Returns a Surface """
        
        try:
            return self.terrain_assets[char]
        except:
            # return default red block since tile_image mapping was not found
            return self.DEFAULT_TILE
        
        
        
    def get_object(self, name):
        """ Gets the object image associated with the given object name.
        If no tile image is found for the specified name, return a 
        default large, blue object. 
        
        Returns a Surface """
        
        try:
            return self.object_assets[name]
        except:
            print 'an object required for this level has not been included in the \
                asset list for the game'
            sys.exit()
    
        
        
        
        