'''
Created on 2011-01-30

@author: Chris
'''

import pygame
from loadtools import *


class NPC(pygame.sprite.Sprite):
    '''
    the object that represents the player-controlled character
    in the game
    '''
    def __init__(self, color, initial_position, sprite_imagename):
        '''
        Constructor
        '''
        # this number is expressed as pixels
        self.STRIDE_LENGTH =5
            
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        
        #set sprite animation images
        self.anim_frames = load_sliced_sprites(32, 50, sprite_imagename)
        self.total_walk_anim_frames = len(self.anim_frames) - 1
        # Set player attributes
        
        #movement speed is measured in pixels traveled per second
        self.movement_speed = 100
        self.x_movement = 0;
        self.y_movement = 0;
        
        #keep track of movement (in pixels) since last step animation frame 
        self.horiz_stride_distance = 0
        
        # Create the image that will be displayed and fill it with the
        # right color.
        self.horiz_walk_anim_frame = 0
        self.image = self.anim_frames[self.horiz_walk_anim_frame]
        #self.image.fill(color)

        # Make our top-left corner the passed-in location.
        self.rect = self.image.get_rect()
        self.rect.topleft = initial_position

    
    
    def update(self):
        newpos = (self.rect.topleft[0] + self.x_movement, \
                  self.rect.topleft[1] + self.y_movement)
        self.rect.topleft = newpos
        
        #calculate distance traveled since current animation frame loaded
        self.horiz_stride_distance += math.fabs(self.x_movement)
        
        #check to see if next frame in animation should be loaded
        if self.horiz_stride_distance > self.STRIDE_LENGTH > 0:
            #update the current walk animation frame
            frames_to_advance = self.horiz_stride_distance/self.STRIDE_LENGTH
            self.horiz_walk_anim_frame = int(round((self.horiz_walk_anim_frame + \
                                    frames_to_advance) % \
                                     self.total_walk_anim_frames))
            #DEBUG: print self.horiz_walk_anim_frame, self.total_walk_anim_frames
            
            #update self.horiz_stride_distance to the proper amount due to 
            #a possible walk animation frame change
            self.horiz_stride_distance = self.horiz_stride_distance % \
                self.STRIDE_LENGTH
                
            self.image = self.anim_frames[self.horiz_walk_anim_frame]
            
            
            
    def horiz_velocity(self, x_displacement):
        if (x_displacement < 0 and x_displacement >= 0) or \
            (x_displacement < 0 and self.x_movement >= 0):
            "player's horizontal direction changed"
            self.image = pygame.transform.flip(self.image, 1, 0)
        
        "update player's horizontal displacement"
        self.x_movement = x_displacement
                            