'''
Created on 2011-01-17

@author: Chris
'''

import pygame
from pygame.locals import *
from loadtools import *
import math

class Player(pygame.sprite.Sprite):
    '''
    the object that represents the player-controlled character
    in the game
    '''
    def __init__(self, color, initial_position, horiz_sprite_imagename, \
                 up_vert_sprite_imagename,down_vert_sprite_imagename):
        '''
        Constructor
        '''            
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        
        self.FRAME_BOTTOM = 450
        # this number is expressed as pixels
        self.STRIDE_LENGTH =3
        
        #the amount of money that the player has
        self.money = 25000
        
        
        #set sprite animation images for horizontal movement
        left_horiz_anim_frames = load_sliced_sprites(36,44,horiz_sprite_imagename)
        right_horiz_anim_frames = []
        
        for frame in left_horiz_anim_frames:
            right_horiz_anim_frames.append(pygame.transform.flip(frame,1,0))
        
        self.horiz_anim_frames_set = [left_horiz_anim_frames,right_horiz_anim_frames]
        # at the start of the game, player is facing left
        self.horiz_frames_set_index = 0
        
        
        #set sprite animation images for vertical movement
        up_vert_anim_frames = load_sliced_sprites(36,44,up_vert_sprite_imagename)
        down_vert_anim_frames = load_sliced_sprites(36,44,down_vert_sprite_imagename)
        
        self.vert_anim_frames_set = [up_vert_anim_frames,down_vert_anim_frames]
        # at the start of the game, player is facing up
        self.vert_frames_set_index = 0
                
        
        self.total_walk_anim_frames = len(left_horiz_anim_frames) - 1
        # Set player attributes
        
        #movement speed is measured in pixels traveled per second
        self.movement_speed = 100
        self.x_movement = 0;
        self.y_movement = 0;
        
        #keep track of movement (in pixels) since last step animation frame 
        self.horiz_stride_distance = 0
        self.vert_stride_distance = 0
        
        # Create the image that will be displayed and fill it with the
        # right color.
        self.horiz_walk_anim_frame = 0
        self.vert_walk_anim_frame = 0
        self.image = \
            self.horiz_anim_frames_set[self.vert_frames_set_index][self.horiz_walk_anim_frame]
        #self.image.fill(color)

        # Make our top-left corner the passed-in location.
        self.rect = self.image.get_rect()
        self.rect.topleft = initial_position

    
    
    def update(self):
        newpos_x = self.rect.topleft[0] + self.x_movement
        newpos_y = self.rect.topleft[1] + self.y_movement
        
        #make sure that player can't leave the playing area
        # check x-coord
        if newpos_x < 0:
            newpos_x = 0
        elif newpos_x > (800 - self.rect.width):
            newpos_x = (800 - self.rect.width)
            
        #check y-coord
        if newpos_y < 0:
            newpos_y = 0
        elif newpos_y > (450 - self.rect.height):
            newpos_y = (450 - self.rect.height)
            
        # player should now be within the game frame
        self.rect.topleft = (newpos_x, newpos_y)
        
        
        if self.x_movement != 0:
            #handle horizontal animation
            #calculate distance traveled since current animation frame loaded
            self.horiz_stride_distance += math.fabs(self.x_movement)
        
            #check to see if next frame in animation should be loaded
            if self.horiz_stride_distance > self.STRIDE_LENGTH:
                #update the current walk animation frame
                frames_to_advance = self.horiz_stride_distance/self.STRIDE_LENGTH
                self.horiz_walk_anim_frame = int(round((self.horiz_walk_anim_frame + \
                                    frames_to_advance) % \
                                     self.total_walk_anim_frames))
                #DEBUG: print self.horiz_walk_anim_frame, self.total_walk_anim_frames
            
                #update self.horiz_stride_distance to the proper amount due to 
                #a possible walk animation frame change
                self.horiz_stride_distance = self.horiz_stride_distance % \
                    self.STRIDE_LENGTH
                
                self.image = \
                    self.horiz_anim_frames_set[self.horiz_frames_set_index][self.horiz_walk_anim_frame]
        
        
        elif self.y_movement != 0:
            #handle vertical animation
            #calculate distance traveled since current animation frame loaded
            self.vert_stride_distance += math.fabs(self.y_movement)
        
            #check to see if next frame in animation should be loaded
            if self.vert_stride_distance > self.STRIDE_LENGTH:
                #update the current walk animation frame
                frames_to_advance = self.vert_stride_distance/self.STRIDE_LENGTH
                self.vert_walk_anim_frame = int(round((self.vert_walk_anim_frame + \
                                    frames_to_advance) % \
                                     self.total_walk_anim_frames))
                #DEBUG: print self.horiz_walk_anim_frame, self.total_walk_anim_frames
            
                #update self.horiz_stride_distance to the proper amount due to 
                #a possible walk animation frame change
                self.vert_stride_distance = self.vert_stride_distance % \
                    self.STRIDE_LENGTH
                
                self.image = \
                    self.vert_anim_frames_set[self.vert_frames_set_index][self.vert_walk_anim_frame]
                        
            
    def horiz_velocity(self, x_displacement):
        # check for change in player horizontal direction and change frame set
        # appropriately
        if x_displacement < 0:
            self.horiz_frames_set_index = 0
        else:
            self.horiz_frames_set_index = 1
        
        "update player's horizontal displacement"
        self.x_movement = x_displacement
        
        
        
        
        
    def vert_velocity(self, y_displacement):
         # check for change in player vertical direction and change frame set
        # appropriately
        if y_displacement < 0:
            self.vert_frames_set_index = 0
        else:
            self.vert_frames_set_index = 1        
        "update player's vertical displacement"
        self.y_movement = y_displacement
                            